package org.maachang.mailslot ;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

/**
 * Nativeメールスロット.
 *  
 * @version 2008/03/10
 * @author  masahito suzuki
 * @since  MailSlot 1.00
 */
class NativeMailSlot {
    private static boolean initFlag = false ;
    private static final boolean LIB_COPY = true ;
    static {
        String lib = null ;
        switch( Os.getInstance().getOS() ) {
            case Os.OS_WINNT :
                lib = "MailSlot.dll" ;
                System.load( targetDynamincLib( LIB_COPY,lib ) ) ;
                initFlag = true ;
                break ;
            default :
                initFlag = false ;
                break ;
        }
    }
    
    private static final String targetDynamincLib( boolean mode,String lib ) {
        String sp = System.getProperty( "file.separator" ) ;
        File targetDir = null ;
        if( System.getProperty( "org.maachang.share.dir" ) == null ) {
            targetDir = new File( new StringBuilder().
                append( System.getProperty( "user.home" ) ).
                append( sp ).
                append( ".lib_work" ).
                toString() );
        }
        else {
            targetDir = new File( System.getProperty( "org.maachang.mailslot.dir" ) );
        }
        if( targetDir.exists() == false ) {
            targetDir.mkdirs() ;
        }
        File outFile = new File( targetDir,lib );
        if( mode == true ) {
            InputStream is = new BufferedInputStream( Thread.currentThread().getContextClassLoader().getResourceAsStream(
                "org/maachang/mailslot/" + lib ) ) ;
            if( isLibFile( outFile,is ) ) {
                try{
                    OutputStream os = new BufferedOutputStream( new FileOutputStream( outFile ) ) ;
                    try{
                        try{
                            for(;;) {
                                int n = is.read();
                                if( n <= -1 ) {
                                    break;
                                }
                                os.write( n );
                            }
                            os.flush() ;
                        }
                        finally{
                            is.close();
                            is = null ;
                        }
                    }
                    finally{
                        os.close();
                        os = null ;
                    }
                }
                catch( Exception e ) {
                    outFile = null ;
                }
            }
            if( is != null ) {
                try {
                    is.close() ;
                } catch( Exception e ) {
                }
                is = null ;
            }
        }
        if( outFile != null ) {
            return outFile.getAbsolutePath() ;
        }
        return null ;
    }
    
    private static final boolean isLibFile( File f,InputStream in ) {
        boolean ret = false ;
        try {
            if( f.exists() == false ) {
                ret = true ;
            }
            else {
                int len = ( int )f.length() ;
                ret = ( len <= 0 || len != in.available() ) ;
            }
        } catch( Exception e ) {
            ret = true ;
        }
        return ret ;
    }
    
    public NativeMailSlot() throws Exception {
        switch( Os.getInstance().getOS() ) {
            case Os.OS_WINNT :
                break ;
            default :
                throw new IOException( "MailSlotは、現在動作中のOSでは対応していません" ) ;
        }
        if( initFlag == false ) {
            throw new IOException( "MailSlotは、現在動作中のOSでは対応していません" ) ;
        }
    }
    
    protected void finalize() throws Exception {
        destroyMailSlot() ;
    }
    
    public void openMailSlot( String name ) throws Exception {
        if( name == null || ( name = name.trim() ).length() <= 0 ) {
            throw new IllegalArgumentException( "引数は不正です" ) ;
        }
        if( initMailSlot( convertBinary( name ) ) <= -1 ) {
            throw new IOException( "メールスロット作成時にエラーが発生しました" ) ;
        }
    }
    
    public void closeMailSlot() {
        destroyMailSlot() ;
    }
    
    public byte[] readMailSlot() {
        return receiveMailSlot() ;
    }
    
    public void writeMailSlot( String comp,String name,byte[] data ) {
        if( name == null || ( name = name.trim() ).length() <= 0 ||
            data == null || data.length <= 0 ) {
            throw new IllegalArgumentException( "引数は不正です" ) ;
        }
        if( comp == null || ( comp = comp.trim() ).length() <= 0 ) {
            sendMailSlot( new byte[1],convertBinary( name ),cbinary( data ) ) ;
        }
        else {
            sendMailSlot( convertBinary( comp ),convertBinary( name ),cbinary( data ) ) ;
        }
    }
    
    private byte[] convertBinary( String string ) {
        byte[] b = string.getBytes() ;
        int len = b.length ;
        byte[] ret = new byte[ len+1 ] ;
        System.arraycopy( b,0,ret,0,len ) ;
        b = null ;
        ret[ len ] = 0 ;
        return ret ;
    }
    
    private byte[] cbinary( byte[] b ) {
        int len = b.length ;
        byte[] ret = new byte[ len+1 ] ;
        System.arraycopy( b,0,ret,0,len ) ;
        b = null ;
        ret[ len ] = 0 ;
        return ret ;
    }
    
    protected native int initMailSlot( byte[] name ) ;
    protected native void destroyMailSlot() ;
    protected native byte[] receiveMailSlot() ;
    protected native void sendMailSlot( byte[] comp,byte[] name,byte[] data ) ;
    
}
